package imageMedd;

import imageMedd.app.ImageDisplayApp;
import imageMedd.transformers.TransformerInit;
import imageMedd.transformers.TransformerVisitor;
import imageMedd.transformers.compositors.CompositeParallelizer;
import imageMedd.transformers.compositors.PixelCompositor;
import imageMedd.transformers.warpers.Warper;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.imageio.ImageIO;


public class ImageManager implements TransformerVisitor {
	protected final int imagesNum = 2;
	protected BufferedImage resultImage;
	protected BufferedImage[] image = new BufferedImage[imagesNum];
	protected ImageDisplayApp viewer[] = new ImageDisplayApp[imagesNum];
	protected ImageDisplayApp resultViewer = null;

	private static ImageManager instance = new ImageManager();
	
	public static ImageManager getInstance() {
		return instance;
	}
	
	private ImageManager() {
		super();
		new TransformerInit();
	}
	
	public void handleArguments(String args[], int n) throws IOException {
		image[0] = ImageIO.read(new File(args[1]));
		if (n >= 4) {
			image[1] = ImageIO.read(new File(args[2]));
		}
	}

	public void produceView(int index) {
		if (viewer[index] != null) {
			viewer[index].kill();
		}
		viewer[index] = new ImageDisplayApp(image[index]);
	}

	public void produceResultView() {
		if (resultViewer != null) {
			resultViewer.kill();
		}
		resultViewer = new ImageDisplayApp(resultImage);
	}
	
	public boolean isValidImage(int index) {
		if (image[index]!=null)
			return true;
		return false;
	}

	public ImageManager(BufferedImage im1, BufferedImage im2) {
		super();
		image[0] = im1;
		if (im2 != null) {
			image[1] = im2;
		}
	}

	public void setImage(int index, BufferedImage image) {
		this.image[index] = image;
	}

	public BufferedImage getImage(int index) {
		return this.image[index];
	}

	public BufferedImage getResultImage() {
		return resultImage;
	}
	
	public int imageSlots() {
		return this.imagesNum;
	}
	
	public int countValidImages() {
		int count = 0;
		for (int i = 0; i<this.imagesNum;i++)
			if (image[i]!=null)
				count ++;
		return count;
	}
	
	@Override
	public void visit(Warper w) {
		if (image[0] == null)
			throw new RuntimeException("No Image in slot #0");
		resultImage = w.warp(image[0]);
	}

	@Override
	public void visit(PixelCompositor c) {
		if (image[0] == null)
			throw new RuntimeException("No Image in slot #0");
		if (image[1] == null)
			throw new RuntimeException("No Image in slot #1");
		if (image[0].getWidth() != image[1].getWidth()
				|| image[0].getHeight() != image[1].getHeight()) {
			throw new RuntimeException("Sizes differ");
		}
		CompositeParallelizer p = new CompositeParallelizer(image[0], image[1], c);
		try {
			resultImage = p.execute();
		} catch (InterruptedException e) {
			throw new RuntimeException("Thread interruption.");
		}
	}

	public void writeResult(String filename) throws IOException {
		if (resultImage == null)
			throw new IOException("No result.");
		if (filename==null)
			throw new FileNotFoundException("No such file: "+filename);
		ImageIO.write(resultImage, "png", new File(filename));
	}
	
	public boolean isValidResult() {
		return (resultImage != null);
	}
}
