package imageMedd;


/**
 * @author OpenG 
 * 		   Pixel class is very similar to Color class, yet it has a few
 *         more tweaks. it has component add and mult to insure no channel gets
 *         past 255 or under 0, and it also has arithmetic and compositing support.
 */
public class Pixel {
	private int r;
	private int g;
	private int b;
	private int a;

	public Pixel(int r, int g, int b, int a) {
		if (r > 255)
			this.r = 255;
		this.setR(r);
		this.setG(g);
		this.setB(b);
		this.setA(a);
	}

	public Pixel(Pixel p) {
		this.setRGB(p.getRGB());
	}

	public Pixel(int rgb) {
		this.a = ((rgb & (255 << 24)) >> 24);
		if (this.a < 0)
			this.a += 256;
		this.r = ((rgb & (255 << 16)) >> 16);
		this.g = ((rgb & (255 << 8)) >> 8);
		this.b = (rgb & (255));
	}

	public int getR() {
		return r;
	}

	public void setR(int r) {
		if (r > 255)
			this.r = 255;
		else if (r < 0)
			this.r = 0;
		else
			this.r = r;
	}

	public int getG() {
		return g;
	}

	public void setG(int g) {
		if (g > 255)
			this.g = 255;
		else if (g < 0)
			this.g = 0;
		else
			this.g = g;
	}

	public int getB() {
		return b;
	}

	public void setB(int b) {
		if (b > 255)
			this.b = 255;
		else if (b < 0)
			this.b = 0;
		else
			this.b = b;
	}

	public int getA() {
		return a;
	}

	public void setA(int a) {
		if (a > 255)
			this.a = 255;
		else if (a < 0)
			this.a = 0;
		else
			this.a = a;
	}

	public int getRGB() {
		return ((this.b | (this.a) << 24) | (this.r << 16)) | (this.g << 8);
	}

	public void setRGB(int rgb) {
		this.a = ((rgb & (255 << 24)) >> 24);
		if (this.a < 0)
			this.a += 256;
		this.r = ((rgb & (255 << 16)) >> 16);
		this.g = ((rgb & (255 << 8)) >> 8);
		this.b = (rgb & (255));
	}

	@Override
	public String toString() {
		return "Pixel [r=" + r + ", g=" + g + ", b=" + b + ", a=" + a + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + a;
		result = prime * result + b;
		result = prime * result + g;
		result = prime * result + r;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Pixel other = (Pixel) obj;
		if (a != other.a)
			return false;
		if (b != other.b)
			return false;
		if (g != other.g)
			return false;
		if (r != other.r)
			return false;
		return true;
	}

	public static Pixel mult(Pixel p, double f) {
		return new Pixel((int) (p.r * f), (int) (p.g * f), (int) (p.b * f),
				(int) (p.a * f));
	}

	public static int addComponent(int c1, int c2) {
		if (c1 + c2 > 255)
			return 255;
		if (c1 + c2 < 0)
			return 0;
		return c1 + c2;
	}

	public static Pixel add(Pixel p, Pixel q) {
		return new Pixel(addComponent(p.r, q.r), addComponent(p.g, q.g),
				addComponent(p.b, q.b), addComponent(p.a, q.a));
	}

	public static Pixel xor(Pixel p, Pixel q) {
		double asrc = (double) p.a / 255, adst = (double) q.a / 255;
		Pixel res = add(mult(p, (asrc * (1 - adst))), mult(q,
				(adst * (1 - asrc))));
		res.a = (int) (255.0 * ((asrc * (1 - adst)) + (adst * (1 - asrc))));
		return res;
	}

	public static Pixel over(Pixel p, Pixel q) {
		double asrc = (double) p.a / 255, adst = (double) q.a / 255;
		Pixel res = add(
				add(mult(p, asrc * adst), mult(p, (asrc * (1 - adst)))), mult(
						q, (adst * (1 - asrc))));
		res.a = (int) (255.0 * (asrc * adst + (asrc * (1 - adst)) + (adst * (1 - asrc))));
		return res;
	}

	public static Pixel in(Pixel p, Pixel q) {
		double asrc = (double) p.a / 255, adst = (double) q.a / 255;
		Pixel res = mult(p, asrc * adst);
		res.a = (int) (255.0 * asrc * adst);
		return res;
	}

	public static Pixel atop(Pixel p, Pixel q) {
		double asrc = (double) p.a / 255, adst = (double) q.a / 255;
		Pixel res = add(mult(p, asrc * adst), mult(q, (adst * (1 - asrc))));
		res.a = (int) (255.0 * asrc * adst + 255.0 * (adst * (1 - asrc)));
		return res;
	}
	
	public static double distance(Pixel p, Pixel q) {
		int difR = p.getR() - q.getR(),
		difG = p.getG() - q.getG(),
		difB = p.getB() - q.getB(),
		difA = p.getA() - q.getA();
		difR*=difR;
		difG*=difG;
		difB*=difB;
		difA*=difA;
		return Math.sqrt(difR+difG+difB+difA);
	}
	
	public static double colorDistance(Pixel p, Pixel q) {
		int difR = p.getR() - q.getR(),
		difG = p.getG() - q.getG(),
		difB = p.getB() - q.getB();
		difR*=difR;
		difG*=difG;
		difB*=difB;
		return Math.sqrt(difR+difG+difB);
	}
}
