package imageMedd.app;

import imageMedd.ImageManager;
import imageMedd.transformers.Transformer;

import java.awt.Button;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Label;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.MenuItem;
import java.awt.MenuShortcut;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Map;


public class Apple {
	// System Variables
	// Java's whatever serial number
	private static final long serialVersionUID = 1L;
	// App size
	private static final int appWidth = 657, appHeight = 250, textFieldWidth = 70;
	// Control label color on events
	public static final String successColor = "#66FF66", failColor = "#FF6666";

	// GUI objects
	// Input fields for file names and parameters
	private static final int paramNum = 5;
	private final TextField file[]= new TextField[ImageManager.getInstance().imageSlots()], out = new TextField(70),
			parameter[] = new TextField[paramNum];
	// Control label, displaying errors and such
	protected final Label control = new Label("GO                        :)");
	// The main frame
	protected final Frame mainframe = new Frame("Image meddling app");
	// The image manager - holding images and performing actions on them when
	// specified
	final ImageManager im = ImageManager.getInstance();
	// Help
	private PublicHelp publicHelp = null;

	Apple() {
		// Looks
		mainframe.setBackground(Color.decode("#DDFFFF"));
		mainframe.setSize(appWidth, appHeight);
		control.setBackground(Color.decode("#00DDFF"));
		// App responds to windows' X button in the corner
		mainframe.setLayout(new FlowLayout(FlowLayout.LEFT));
		mainframe.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		// Insert two load fields for two images
		for (int i = 0;i < ImageManager.getInstance().imageSlots(); i++) {
			file[i] = new TextField(textFieldWidth);
			addLoadField("Load!", i, file[i]);
		}
		// Add a button for each possible action
		for (int i = 0; i < paramNum; i++)
			addParameterField();
		addActionButtons();
		// Add another text field to specify output file
		addFlushField(out);
		// Add menu bar
		putMenus();
		// Add control label
		mainframe.add(control);
		mainframe.setVisible(true);
	}

	private static int count = 0;

	private void addParameterField() {
		if (count >= paramNum)
			throw new RuntimeException("Too many parameter fields");
		mainframe.add(new Label("Parameter " + count + ":"));
		parameter[count] = new TextField();
		mainframe.add(parameter[count]);
		count++;
	}

	private void addLoadField(String label, int ind, TextField field) {
		mainframe.add(new Label("Load image" + ind + ":"));
		mainframe.add(field);
		addLoadButton("Load!", ind, field);
	}

	private void addLoadButton(String string, int index, TextField text) {
		// Add a load button next to the input field
		Button b1 = new Button(string);
		// Preview Button
		Button b2 = new Button("Preview");
		// What will happen when hitting it:
		b1.addActionListener(new LoadImageButtonActionListener(text, index,
				this));
		b2.addActionListener(new PreviewActionListener(index, this));
		mainframe.add(b1);
		mainframe.add(b2);
	}

	private void addActionButtons() {
		for (Map.Entry<String, Transformer> e : Transformer.transformers.entrySet()) {
			Button b = new Button(e.getKey());
			b.addActionListener(new TransformButtonActionListener(e.getValue(), im, this));
			mainframe.add(b);
		}
	}

	private void addFlushField(final TextField res) {
		mainframe.add(new Label("Flush result: "));
		mainframe.add(res);
		Button b = new Button("Flush!");
		b.addActionListener(new ActionListener() {
			TextField result = res;

			@Override
			public void actionPerformed(ActionEvent arg0) {
				try {
					if (result.getText().startsWith("#")
							&& (result.getText().length() == 2)
							&& (result.getText().charAt(1) - '0' < im
									.imageSlots() && result.getText()
									.charAt(1) - '0' >= 0)) {
						im.setImage(result.getText().charAt(1) - '0', im
								.getResultImage());
						controlSays("Result flushed to slot "
								+ result.getText(), successColor);
					} else {
						im.writeResult(result.getText());
						controlSays("Congrats, you got a new picture!",
								successColor);
					}
				} catch (Exception e) {
					controlSays("Flush: " + e.getMessage(), failColor);
				}
			}
		});
		mainframe.add(b);
		Button b2 = new Button("Preview");
		b2.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (ImageManager.getInstance().isValidResult()) {
					ImageManager.getInstance().produceResultView();
				} else
					controlSays("No image in result", failColor);
			}
		});
		mainframe.add(b2);
	}

	private void putMenus() {
		ActionListener fileExitAL = new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				System.exit(0);
			}
		};
		MenuBar menu = new MenuBar();
		Menu file = new Menu("File");
		MenuItem fileExit = new MenuItem("Exit",
				new MenuShortcut(KeyEvent.VK_Q));
		fileExit.addActionListener(fileExitAL);
		menu.add(file);
		file.add(fileExit);
		Menu help = new Menu("Help");
		MenuItem helpHelp = new MenuItem("Help",
				new MenuShortcut(KeyEvent.VK_H));
		helpHelp.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				if (publicHelp != null)
					publicHelp.kill();
				publicHelp = new PublicHelp();
			}
		});
		help.add(helpHelp);
		menu.add(help);
		mainframe.setMenuBar(menu);
	}

	private final int CTRLpixelsPerLetter = 6, CTRLheight = 15;

	public void controlSays(String message, String colorInd) {
		control.setSize(message.length() * CTRLpixelsPerLetter, CTRLheight);
		control.setBackground(Color.decode(colorInd));
		control.setText(message);
	}

	private static RuntimeException PARSING_ERROR = new RuntimeException(
			"Parameters error.");

	public int[] getParam() {
		int p[] = new int[paramNum];
		for (int i = 0; i < paramNum; i++) {
			try {
				if (parameter[i].getText().length() == 0)
					p[i] = 1;
				else if (parameter[i].getText().startsWith("0x")) {
					p[i] = parseHex(parameter[i].getText().substring(2));
				} else
					p[i] = Integer.parseInt(parameter[i].getText());
			} catch (RuntimeException e) {
				throw PARSING_ERROR;
			}
		}
		return p;
	}

	private int parseHex(String hex) {
		if (hex.length() > 8)
			throw new RuntimeException("Parameter too big.");
		Long parsed = Long.parseLong(hex, 16);
		return parsed.intValue();
	}

	public static void main(String args[]) {
		new Apple();
	}
}
