package imageMedd.transformers.compositors;

import imageMedd.Pixel;

import java.awt.image.BufferedImage;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;


public class CompositeParallelizer {
	protected final BufferedImage i1, i2;
	protected final BufferedImage result;
	protected final PixelCompositor t;
	protected final static int nThreads = 1;
	
	private static final long timeout = 5;
	private static final TimeUnit unit = TimeUnit.MINUTES;
	
	protected class CompositeRunner implements Runnable {
		private final int x, y; 
		public CompositeRunner(int x, int y) {
			this.x = x;
			this.y = y;
		}
		
		@Override
		public void run() {
			Pixel p = new Pixel(i1.getRGB(x, y));
			Pixel q = new Pixel(i2.getRGB(x, y));
			result.setRGB(x, y, t.composite(p, q).getRGB());
		}
	}
	
	public CompositeParallelizer(BufferedImage image1, BufferedImage image2, PixelCompositor t) {
		i1 = image1;
		i2 = image2;
		result = new BufferedImage(i1.getWidth(), i1.getHeight(), BufferedImage.TYPE_INT_ARGB);
		this.t = t;
	}
	
	public BufferedImage execute() throws InterruptedException {
		ExecutorService e = Executors.newFixedThreadPool(nThreads);
		for (int i = 0; i<i1.getWidth();i++)
			for (int j = 0; j<i1.getHeight();j++)
				e.execute(new CompositeRunner(i, j));
		e.shutdown();
		e.awaitTermination(timeout, unit);
		return result;
	}
}
