package imageMedd.transformers.warpers;

import imageMedd.Pixel;
import imageMedd.transformers.TransformerVisitor;

import java.awt.image.BufferedImage;
import java.util.Random;


public class OmferMosaic extends Warper {
	private static final String key = "-mosaic";
	private int squareSize = 6;

	public OmferMosaic() {
		super();
	}

	@Override
	public BufferedImage warp(BufferedImage image1) {
		BufferedImage result = new BufferedImage(image1.getWidth(), image1
				.getHeight(), BufferedImage.TYPE_INT_ARGB);
		for (int i = 0; i < image1.getWidth(); i += squareSize) {
			for (int j = 0; j < image1.getHeight(); j += squareSize) {
				int r = 0, g = 0, b = 0, a = 0;
				for (int k = 0; k < squareSize && i + k < image1.getWidth(); k++) {
					for (int w = 0; w < squareSize
							&& j + w < image1.getHeight(); w++) {
						Pixel cur = new Pixel(image1.getRGB((k + i), (j + w)));
						r += cur.getR();
						g += cur.getG();
						b += cur.getB();
						a += cur.getA();
					}
				}
				r /= (squareSize * squareSize);
				g /= (squareSize * squareSize);
				b /= (squareSize * squareSize);
				a /= (squareSize * squareSize);
				Pixel avg = new Pixel(r, g, b, 255);
				if (a < 128)
					avg.setA(0);
				for (int k = 0; k < squareSize && i + k < image1.getWidth(); k++) {
					for (int w = 0; w < squareSize
							&& j + w < image1.getHeight(); w++) {
						result.setRGB((k + i), (j + w), avg.getRGB());
					}
				}
			}
		}
		Random r = new Random();
		for (int i = 0; i < image1.getWidth(); i += squareSize) {
			for (int j = 2; j < image1.getHeight(); j += squareSize) {
				int inout = (r.nextInt() % 3 == 0) ? -1 : 1;
				for (int k = 0; k < squareSize && i + k < image1.getWidth(); k++) {
					if (k < 5) {
						Pixel p = Pixel.mult(
								new Pixel(result.getRGB(k + i, j)), 0.5);
						p.setA(255);
						result.setRGB(k + i, j, p.getRGB());
					} else {
						Pixel p = Pixel.mult(
								new Pixel(result.getRGB(k + i, j)), 0.75);
						p.setA(255);
						result.setRGB(k + i, j, p.getRGB());
					}
					if ((j + inout - image1.getHeight() < 0)
							&& (j + inout >= 0)) {
						Pixel p = Pixel.mult(new Pixel(result.getRGB(k + i, j
								+ inout)), 0.75);
						p.setA(255);
						result.setRGB(k + i, j + inout, p.getRGB());
					}
				}
			}
		}

		for (int i = 2; i < image1.getWidth(); i += squareSize) {
			for (int j = 0; j < image1.getHeight(); j += squareSize) {
				for (int k = 0; k < squareSize && j + k < image1.getHeight(); k++) {
					if (k != 4) {
						Pixel p = Pixel.mult(
								new Pixel(result.getRGB(i, j + k)), 0.75);
						p.setA(255);
						result.setRGB(i, j + k, p.getRGB());
					}
				}
			}
		}

		return result;
	}

	@Override
	public void accept(TransformerVisitor v) {
		v.visit(this);
	}

	@Override
	public void setParam(int[] param) {
		if (param.length < 1)
			this.squareSize = 6;
		else {
			if (param[0] < 1) {
				throw new RuntimeException("Unable to create mosaice with this square size.");
			}
			this.squareSize = param[0];
		}
			
	}
	
	@Override
	public String getString() {
		return key;
	}

	@Override
	public String help() {
		return "Creates a primitive mosaic of the image. Takes square size on parameter 0.";
	}
}
