package imageMedd.transformers.warpers;

import imageMedd.Pixel;
import imageMedd.transformers.TransformerVisitor;

import java.awt.image.BufferedImage;


public class ReplaceColor extends Warper {
	private static final String key = "-replace";
	private static int color1 = 0xFFFFFFFF, color2 = 0, rad = 0;
	private static boolean regardAlpha = false;

	public ReplaceColor() {
		super();
	}

	@Override
	public BufferedImage warp(BufferedImage image1) {
		BufferedImage result = new BufferedImage(image1.getWidth(), image1
				.getHeight(), BufferedImage.TYPE_INT_ARGB);
		Pixel dest = new Pixel(color1);
		for (int i = 0; i < result.getWidth(); i++)
			for (int j = 0; j < result.getHeight(); j++) {
				int rgb = image1.getRGB(i, j);
				Pixel p = new Pixel(rgb);
				if ((regardAlpha && Pixel.distance(p, dest) <= rad) || (!regardAlpha && Pixel.colorDistance(p, dest) <= rad))
					result.setRGB(i, j, color2);
				else
					result.setRGB(i, j, rgb);
			}
		return result;
	}

	@Override
	public void accept(TransformerVisitor v) {
		v.visit(this);
	}

	@Override
	public String getString() {
		return key;
	}

	@Override
	public void setParam(int[] param) {
		switch (param.length) {
		case 0:
			throw new RuntimeException("Colors unknown.");
		case 1:
			color1 = param[0];
			color2 = 0xFFFFFFFF;
			rad = 0;
			break;
		case 2:
			color1 = param[0];
			color2 = param[1];
			rad = 0;
			break;
		default:
			regardAlpha = (param[3] == 0);
		case 3:
			if (param[2] < 0)
				throw new RuntimeException("No negative radiuses.");
			color1 = param[0];
			color2 = param[1];
			rad = param[2];
		}
	}
	
	@Override
	public String help() {
		return "Replaces every color, which is close enough to color 1, to color 2. Gets color 1 at parameter 0, color 2 at parameter 1, and the distance at parameter 2.<br>Put 0 in parameter 3 to calculate distance with alpha channel.";
	}
}
