package imageMedd.transformers.warpers;

import imageMedd.Pixel;
import imageMedd.transformers.TransformerVisitor;

import java.awt.image.BufferedImage;


public class Stroker extends Warper {
	protected static final String key = "-stroke";

	public Stroker() {
		super();
	}

	private static int diagSize = 16;
	private static double plus = 0.05;
	private static boolean shader = false;

	@Override
	public BufferedImage warp(BufferedImage image1) {
		BufferedImage res = new BufferedImage(image1.getWidth(), image1
				.getHeight(), BufferedImage.TYPE_INT_ARGB);
		for (int i = 0; i < image1.getWidth(); i++)
			for (int j = 0; j < image1.getHeight(); j++) {
				res.setRGB(i, j, image1.getRGB(i, j));
			}
		for (int i = 0 + diagSize; i < image1.getWidth() - diagSize; i++)
			for (int j = 0 + diagSize; j < image1.getHeight() - diagSize; j++)
				for (int k = -diagSize * ((i + j) % 2); k < diagSize
						* ((i + j + 1) % 2); k++) {
					if (!shader || (new Pixel(image1.getRGB(i, j)).getA() != 0 && new Pixel(image1.getRGB(i + k, j + k)).getA() == 0))
						res.setRGB(i + k, j + k, Pixel.add(
								Pixel.mult(new Pixel(res.getRGB(i, j)), plus),
								new Pixel(res.getRGB(i + k, j + k))).getRGB());
				}
		return res;
	}

	@Override
	public void accept(TransformerVisitor v) {
		v.visit(this);
	}

	@Override
	public String getString() {
		return key;
	}

	@Override
	public void setParam(int param[]) {
		if (param.length > 0)
			diagSize = param[0];
		if (param.length > 1)
			plus = param[1] / 100.0;
		if (param.length > 2)
			shader = (param[2] == 0);
	}

	@Override
	public String help() {
		return "Draws part of each pixel over adjacant pixels. Gets stretch length at parameter 0 and stretch strength at parameter 1.<br>Enter 0 at parameter 2 to use this as a shader.";
	}
}
