package imageMedd.transformers.warpers;

import imageMedd.Pixel;
import imageMedd.transformers.TransformerVisitor;

import java.awt.image.BufferedImage;


public class TripleShadow extends Warper {
	private static final String key = "-tri";
	private static int Xdiff = 0, Ydiff = 0;
	private static float precentage = 0.3f;

	public TripleShadow() {
		super();
	}

	@Override
	public BufferedImage warp(BufferedImage image1) {
		BufferedImage result = new BufferedImage(image1.getWidth(), image1
				.getHeight(), BufferedImage.TYPE_INT_ARGB);
		for (int i = 0; i < image1.getWidth(); i++)
			for (int j = 0; j < image1.getHeight(); j++)
				result.setRGB(i, j, image1.getRGB(i, j));
		for (int i = 0; i < result.getWidth(); i++)
			for (int j = 0; j < result.getHeight(); j++) {
				Pixel p = new Pixel(image1.getRGB(i, j));
				int avg = (int) ((p.getR() + p.getG() + p.getB()) / 3 * precentage);
				if (i + Xdiff < result.getWidth() && j + Ydiff < result.getHeight() && i + Xdiff >= 0 && j + Ydiff >= 0)
					result.setRGB(i + Xdiff, j + Ydiff, Pixel.add(
							new Pixel(result.getRGB(i + Xdiff, j + Ydiff)),
							new Pixel(avg, 0, 0, avg)).getRGB());
				if (i + 2 * Xdiff < result.getWidth()
						&& j + 2 * Ydiff < result.getHeight()
						&& i + 2 * Xdiff >= 0 && j + 2 * Ydiff >= 0)
					result.setRGB(i + 2 * Xdiff, j + 2 * Ydiff, Pixel.add(
							new Pixel(result.getRGB(i + 2 * Xdiff, j + 2*Ydiff)),
							new Pixel(0, avg, 0, avg)).getRGB());
				if (i + 3 * Xdiff < result.getWidth()
						&& j + 3 * Ydiff < result.getHeight()
						&& i + 3 * Xdiff >= 0 && j + 3 * Ydiff >= 0)
					result.setRGB(i + 3 * Xdiff, j + 3 * Ydiff, Pixel.add(
							new Pixel(result.getRGB(i + 3 * Xdiff, j + 3*Ydiff)),
							new Pixel(0, 0, avg, avg)).getRGB());
			}
		return result;
	}

	@Override
	public void accept(TransformerVisitor v) {
		v.visit(this);
	}

	@Override
	public String getString() {
		return key;
	}

	@Override
	public void setParam(int param[]) {
		if (param.length > 0)
			Xdiff = param[0];
		if (param.length > 1)
			Ydiff = param[1];
		if (param.length > 2) {
			if (param[2] > 100 || param[2] < 0)
				throw new RuntimeException("Bad precentage.");
			else
				precentage = param[2] / 100.0f;
		}
	}

	@Override
	public String help() {
		return "Adds three different (red, green and blue) shadows. Gets X offset for shadow at parameter 0, Y offset at parameter 1, and shadow srength at parameter 2.";
	}
}
